package com.github.ln12.library

import android.content.Context
import android.net.ConnectivityManager
import android.net.Network
import android.net.NetworkCapabilities
import android.net.NetworkRequest
import android.os.Build
import android.util.Log
import kotlinx.coroutines.flow.MutableStateFlow

// From library com.github.ln-12:multiplatform-connectivity-status:1.1.0
// https://github.com/ln-12/multiplatform-connectivity-status
// Copyright 2021 Lorenzo Neumann
// Edited by davidoskky as here: https://github.com/ln-12/multiplatform-connectivity-status/pull/4
actual class ConnectivityStatus(private val context: Context) {
    actual val isNetworkConnected = MutableStateFlow(false)

    private var connectivityManager: ConnectivityManager? = null
    private val networkCallback = object : ConnectivityManager.NetworkCallback() {
        override fun onAvailable(network: Network) {
            Log.d("Connectivity status", "Connected")
            isNetworkConnected.value = true
        }

        override fun onLost(network: Network) {
            Log.d("Connectivity status", "Disconnected")
            isNetworkConnected.value = false
        }
    }

    actual fun start() {
        try {
            if (connectivityManager == null) {
                connectivityManager = context.getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
            }
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                // API 24 and above
                connectivityManager!!.registerDefaultNetworkCallback(networkCallback)

                val currentNetwork = connectivityManager!!.activeNetwork

                if(currentNetwork == null) {
                    isNetworkConnected.value = false

                    Log.d("Connectivity status", "Disconnected")
                }
            } else {
                // API 23 and below
                val networkRequest = NetworkRequest.Builder().apply {
                    addCapability(NetworkCapabilities.NET_CAPABILITY_INTERNET)
                    addCapability(NetworkCapabilities.NET_CAPABILITY_NOT_RESTRICTED)
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                        addCapability(NetworkCapabilities.NET_CAPABILITY_VALIDATED)
                    }
                }.build()

                connectivityManager!!.registerNetworkCallback(networkRequest, networkCallback)

                val currentNetwork = connectivityManager!!.activeNetworkInfo

                if(currentNetwork == null || (
                    currentNetwork.type != ConnectivityManager.TYPE_ETHERNET &&
                    currentNetwork.type != ConnectivityManager.TYPE_WIFI &&
                    currentNetwork.type != ConnectivityManager.TYPE_MOBILE
                )) {
                    isNetworkConnected.value = false

                    Log.d("Connectivity status", "Disconnected")
                }
            }

            Log.d("Connectivity status", "Started")
        } catch (e: Exception) {
            Log.d("Connectivity status", "Failed to start: ${e.message.toString()}")
            e.printStackTrace()
            isNetworkConnected.value = false
        }
    }

    actual fun stop() {
        connectivityManager?.unregisterNetworkCallback(networkCallback)
        Log.d("Connectivity status", "Stopped")
    }
}